;  DirWide - simple dir lister, written in NASM 0.97
;  Written: March 13; May 9-25 (1.7); May 26-31, 1998 (1.8);
;  http://www.geocities.com/SiliconValley/Peaks/6079
;  nasm -o dw.com dirwide.asm
;  Last updated on 5-31-98  4:39pm
;

%define offset
%define ptr

FNAME equ 1Eh                            ; dta+FNAME = offset of ASCII filename
                                         ;   found via findfirst or findnext
FDATE equ 18h                            ; dta+FDATE = offset of filedate
FTIME equ 16h                            ; dta+FTIME = offset of filetime
FATTR equ 15h                            ; dta+FATTR = offset of file attribute
FSIZEH equ 1Ch                           ; high word of filesize
FSIZEL equ 1Ah                           ; low word of filesize

CR equ 13                                ; carriage return
LF equ 10                                ; linefeed
NULL equ 0                               ; end of strings, meant for use with
                                         ;    my custom print routine

DIR equ 00010000b                        ; file attribute of a directory
ARCHIVE equ 00100000b                    ; file attribute of archive
SYSTEM equ 00000100b                     ; file attribute of system
HIDDEN equ 00000010b                     ; file attribute of hidden
READONLY equ 00000001b                   ; file attribute of readonly

OPT_C equ 10000000b                      ; bitfield of option C (no commas)
OPT_D equ 01000000b                      ; option D (dirs only)
OPT_F equ 00100000b                      ; option F (files only)
OPT_I equ 00010000b                      ; option I (print misc info)
OPT_L equ 00001000b                      ; option L (hide filespec)
OPT_P equ 00000100b                      ; option P (no pause)
OPT_V equ 00000010b                      ; option V (print extra file info)
OPT_Z equ 00000001b                      ; option Z (ignore lead 0's with /V)

;---------------------------[start of main section]----------------------------
; CODE segment
; ASSUME CS:CODE,DS:CODE,ES:CODE

[section .text]
org 100h                                 ; we're making a .COM file

Start:
      cld                                ; clear direction flag
      mov ah,0Fh                         ; check current video mode
      int 10h
      cmp al,3                           ; this program only works with
                                         ;    textmodes 2 and 3
      jz short no_reset_video_mode
      cmp al,2
      jz short no_reset_video_mode
      mov ax,3                           ; if not in textmode 2 or 3,
      int 10h                            ;    switch to textmode 3
no_reset_video_mode:
      call default                       ; set default options
      call parser                        ; get filespec and options
      call scan                          ; add '*.*' to filespec, if needed
      call findfirst                     ; find first file, if possible
      call findnext                      ; find remaining files, if possible
Exit:
      cmp word ptr [numitems],0          ; have we found anything?
      jnz short Adiaux                   ; if so, check if /I is used
      mov si,offset no_files_found
      call print                         ; 'No files found.'
Adiaux:
      test byte ptr [options],OPT_I      ; is option /I used?
      jnz short Adiaux_no_info           ; if not, don't print info
      call extra_info
Adiaux_no_info:
      mov ax,4C00h                       ; quit program
      int 21h
;---------------------------[end of main section]------------------------------

; parser
parser:
       mov si,80h                        ; beginning of cmdline in PSP
       mov di,offset filespec            ; future location of our filespec
       mov bx,di                         ; save DI in BX for later use
       mov ax,NULL
       mov cx,40
       rep stosw                         ; clear out old memory
       mov di,bx                         ; restore DI
parser_space_check:
       inc si
       cmp byte ptr [si],' '
       jz short parser_space_check       ; skip all leading whitespace
       cmp byte ptr [si],CR
       jz short parser_leave2            ; use '*.*' if no filespec entered
       cmp byte ptr [si],'/'
       jz short parser_options
       cmp byte ptr [si],'-'
       jz short parser_options           ; check for options if '/' or '-'
                                         ;    is found on the cmdline
       movsb
       inc byte ptr [fslen]              ; count filespec length for later use
       mov byte ptr [di],NULL            ; put ending mark on string
       dec si
       jmp short parser_space_check      ; go back, check for more options
parser_leave2:
       jmp parser_leave                  ; otherwise, leave here, begin program
parser_options:
       inc si
       cmp byte ptr [si],' '
       jz short parser_options
       cmp byte ptr [si],'?'             ; /?   get help screen
       jz short parser_help
       cmp byte ptr [si],'a'             ; if option isn't a letter of the
       jb short parser_check_again       ;   alphabet, ignore it
       cmp byte ptr [si],'z'
       ja short parser_space_check
       sub byte ptr [si],'a'-'A'         ; convert all options to uppercase
parser_check_again:
       cmp byte ptr [si],'A'
       jb short parser_space_check
       cmp byte ptr [si],'Z'
       ja short parser_space_check
       cmp byte ptr [si],'C'             ; /C no commas
       jz short parser_no_commas
       cmp byte ptr [si],'P'             ; /P no pause
       jz short parser_nopause
       cmp byte ptr [si],'F'             ; /F files only
       jz short parser_files
       cmp byte ptr [si],'D'             ; /D dirs only
       jz short parser_dirs
       cmp byte ptr [si],'I'             ; /I print misc info
       jz short parser_info
       cmp byte ptr [si],'V'             ; /V print extra info
       jz short parser_verbose
       cmp byte ptr [si],'Z'             ; /Z ignore lead zeros in extra info
       jz short parser_leadzero
       cmp byte ptr [si],'L'
       jz short parser_no_filespec
       cmp byte ptr [si],'H'             ; /H help screen
       jz short parser_help
       jmp short parser_space_check      ; check for more options
parser_help:
       mov si,offset help_msg
       call print                        ; print help screen, exit program
       mov ax,4C00h
       int 21h
parser_no_filespec:
       xor byte ptr [options],OPT_L      ; toggle option L
       jmp parser_space_check
parser_leadzero:
       xor byte ptr [options],OPT_Z      ; toggle option Z
       jmp parser_space_check
parser_verbose:
       xor byte ptr [options],OPT_V      ; toggle option V
       mov byte ptr [pausenum],20        ; change number of files to print
                                         ;    before pausing
       jmp parser_space_check
parser_info:
       xor byte ptr [options],OPT_I      ; toggle option I
       jmp parser_space_check
parser_no_commas:
       xor byte ptr [options],OPT_C      ; toggle option C
       jmp parser_space_check
parser_nopause:
       xor byte ptr [options],OPT_P      ; toggle option P
       jmp parser_space_check
parser_files:
       xor byte ptr [options],OPT_F      ; toggle option F
       test byte ptr [options],OPT_D
       jz short parser_not_all
       xor byte ptr [options],OPT_F      ; if using /F and /D, toggle both
       xor byte ptr [options],OPT_D      ;    and just find everything
parser_not_all:
       jmp parser_space_check
parser_leave:
       jmp short parser_ret
parser_dirs:
       xor byte ptr [options],OPT_D      ; if /D, we're looking for dirs
       xor byte ptr [attribute],DIR      ; save attribute for search
       test byte ptr [options],OPT_F
       jz short parser_not_both
       xor byte ptr [options],OPT_D
       xor byte ptr [options],OPT_F
parser_not_both:
       jmp parser_space_check
parser_ret:
       ret

; print_filename
print_filename:
       call add_total                    ; ' files totaling --- bytes'
       mov byte ptr [numlen],0
       inc word ptr [numitems]           ; increase number of items found
       test byte ptr [options],OPT_I
       jnz short print_filename_no_inc
       test byte ptr [options],OPT_F
       jnz short print_filename_inc_numfiles
       test byte ptr [dta+FATTR],DIR     ; if dir found, increase num of dirs
       jz short print_filename_inc_numfiles
       inc word ptr [numdirs]
       jmp short print_filename_no_inc
print_filename_inc_numfiles:
       test byte ptr [dta+FATTR],DIR
       jnz short print_filename_no_inc
       test byte ptr [options],OPT_D
       jnz short print_filename_no_inc
       inc word ptr [numfiles]           ; if file found, increase num of files
print_filename_no_inc:
       test byte ptr [options],OPT_D
       jnz short print_filename_recheck
       test byte ptr [options],OPT_F
       jz short print_filename_continue
print_filename_recheck:
       test byte ptr [options],OPT_F
       jz short print_filename_no_opt_f
       test byte ptr [dta+FATTR],DIR
       jz short print_filename_continue
print_filename_no_opt_f:
       mov bl,byte ptr [attribute]
       test byte ptr [dta+FATTR],bl
       jnz short print_filename_continue
       dec word ptr [numitems]
print_filename_continue:
       call lowercase                    ; if not a dir, make name lowercase
       test byte ptr [options],OPT_D
       jnz short print_filename_recheck_again
       test byte ptr [options],OPT_F
       jz short print_filename_print_name
print_filename_recheck_again:
       test byte ptr [options],OPT_F
       jz short print_filename_opt_f_not_set
       test byte ptr [dta+FATTR],DIR
       jz short print_filename_print_name
print_filename_opt_f_not_set:
       mov bl,byte ptr [attribute]
       test byte ptr [dta+FATTR],bl
       jz short print_filename_ret
print_filename_print_name:
       mov si,offset dta+FNAME
       call print                        ; print filename
       call spaces
       test byte ptr [dta+FATTR],DIR
       jnz short print_filename_dir_found
       mov dx,word ptr [dta+FSIZEH]
       mov ax,word ptr [dta+FSIZEL]
       call bin2dec
       test byte ptr [options],OPT_C
       jnz short print_filename_no_commas
       call put_commas
print_filename_no_commas:
       test byte ptr [options],OPT_C
       jnz short print_filename_use_buffer
       mov si,offset backup
       jmp short print_filename_print_size
print_filename_use_buffer:
       mov al,NULL
       mov si,offset buffer
       mov di,offset buffer
       mov cx,12
       repnz scasb
       mov ax,offset buffer
       sub di,ax
       xchg di,ax
       dec al
       mov byte ptr [numlen],al
print_filename_print_size:
       call print                        ; print filesize
       jmp short print_filename_ret
print_filename_dir_found:
       mov si,offset dir_msg             ; if dir, print '<dir>'
       call print
       mov cx,13
       call space_out
print_filename_ret:
       test byte ptr [dta+FATTR],DIR
       jnz short print_filename_no_nspace
       call nspaces
print_filename_no_nspace:
       test byte ptr [options],OPT_V
       jz short print_filename_no_opt_v
       test byte ptr [options],OPT_F
       jz short print_filename_check_dir
       test byte ptr [dta+FATTR],DIR
       jz short print_filename_print_crlf
       jmp short print_filename_no_opt_v
print_filename_check_dir:
       test byte ptr [options],OPT_D
       jz short print_filename_print_crlf
print_filename_check_again:
       test byte ptr [dta+FATTR],DIR
       jz short print_filename_no_opt_v
print_filename_print_crlf:
       call verbose                      ; if /V, call verbose
       mov si,offset crlf
       call print                        ; goto next line
print_filename_no_opt_v:
       ret

; findfirst
findfirst:
       mov ah,1Ah                        ; set DTA
       mov dx,offset dta
       int 21h
findfirst_find:
       mov ah,4Eh                        ; find first
       mov cx,11110111b                  ; find all but volume label
       mov dx,offset filespec            ; find 'filespec' (e.g. '*.*')
       int 21h
       jnc short findfirst_print_filename
findfirst_invalid_path:
       cmp ax,3
       jnz short findfirst_no_files_found
       cmp byte ptr [fsbackup],0
       jz short findfirst_path_error
       mov si,offset fsbackup            ; if our intuitive filespec wasn't
       mov di,offset filespec            ;    valid, replace filespec with
       mov cx,40                         ;    original unmodified filespec
       rep movsw                         ;    and try findfirst again
       mov byte ptr [fsbackup],0
       jmp short findfirst_find
findfirst_path_error:
       mov si,offset invalid_path        ; 'Invalid path.'
       call print
       jmp short findfirst_exit
findfirst_no_files_found:
       cmp ax,18
       jnz short findfirst_print_filename
       mov si,offset no_files_found      ; 'No files found.'
       call print
       jmp short findfirst_exit
findfirst_print_filename:
       call print_filespec               ; print filespec ('*.*' of 'dw *.*')
       call print_filename               ; print ASCII filename
       test byte ptr [options],OPT_F
       jnz short findfirst_ret
       test byte ptr [dta+FATTR],DIR
       jnz short findfirst_ret
       cmp word ptr [numitems],1
       jz short findfirst_ret
       call nspaces
findfirst_ret:
       ret
findfirst_exit:
       test byte ptr [options],OPT_I
       jnz short findfirst_no_info
       call extra_info
findfirst_no_info:
       mov ax,4C00h
       int 21h

; findnext
findnext:
       mov ah,4Fh                        ; find next matching file
       mov cx,11110111b                  ; find all but volume label
       mov dx,offset filespec            ; use filespec (e.g. '*.*')
       int 21h
       jnc short findnext_print_filename
findnext_no_files_found:
       cmp ax,18                         ; if no files found, exit
       jz short findnext_goto_ret
findnext_print_filename:
       call print_filename               ; print filename and size (if not dir)
       test byte ptr [dta+FATTR],DIR
       jnz short findnext_crlf
       test byte ptr [options],OPT_D
       jz short findnext_still_rechecking
       test byte ptr [options],OPT_F
       jz short findnext_crlf
findnext_still_rechecking:
       mov ax,word ptr [numitems]
       mov cl,2
       div cl
       or ah,ah
       jz short findnext_crlf            ; on even files, print newline
       test byte ptr [options],OPT_D
       jnz short findnext_work_it
       test byte ptr [options],OPT_F
       jz short findnext_crlf
findnext_work_it:
       mov bl,byte ptr [attribute]
       test byte ptr [dta+FATTR],bl
       jz short findnext_crlf
findnext_crlf:
       test byte ptr [options],OPT_D
       jnz short findnext_more_recheck
       test byte ptr [options],OPT_F
       jz short findnext_ignore_this
       jmp short findnext_more_recheck
findnext_goto_ret:
       jmp short findnext_ret
findnext_more_recheck:
       test byte ptr [options],OPT_F
       jz short findnext_opt_d
       test byte ptr [dta+FATTR],DIR
       jz short findnext_ignore_this
findnext_opt_d:
       mov bl,byte ptr [attribute]
       test byte ptr [dta+FATTR],bl
       jz short findnext_check_for_pause
findnext_ignore_this:
       test byte ptr [options],OPT_V
       jnz short findnext_check_for_pause
       mov ax,word ptr [numitems]
       mov cl,2
       div cl
       or ah,ah
       jnz short findnext_go_home
       mov si,offset crlf
       call print
findnext_check_for_pause:
       cmp word ptr [numitems],0
       jz short findnext_go_home
       test byte ptr [options],OPT_P
       jnz short findnext_go_home
       mov ax,word ptr [numitems]
       mov cl,byte ptr [pausenum]        ; if 'pausenum' num. of files is found,
       div cl                            ;    pause program
       or ah,ah
       jnz short findnext_go_home
       mov ah,8                          ; get input without echo
       int 21h
findnext_go_home:
       jmp findnext                      ; go back and find next file
findnext_ret:
       test byte ptr [options],OPT_V
       jnz short findnext_bye
       mov ax,word ptr [numitems]
       mov cl,2
       div cl
       or ah,ah
       jz short findnext_bye
findnext_print_crlf:
       mov si,offset crlf
       call print
findnext_bye:
       ret

; print
print:                                   ; SI = offset of ASCII string
       mov byte ptr [strlen],0
       mov ah,2
print_load:
       mov dl,byte ptr [si]
       inc si
       cmp dl,NULL                       ; NULL = end of string
       jz short print_load_ret
       int 21h
       inc byte ptr [strlen]             ; store string length for later use
       jmp short print_load
print_load_ret:
       ret

; lowercase
lowercase:                               ; convert filename to lowercase
       test byte ptr [dta+FATTR],DIR     ;    but only if not a dir
       jnz short lowercase_finish_conv
       mov si,offset dta+FNAME           ; dta+FNAME = offset of ASCII filename
       dec si
lowercase_try_again:
       inc si
       cmp byte ptr [si],NULL
       jz short lowercase_finish_conv
       cmp byte ptr [si],'A'
       jb short lowercase_try_again
       cmp byte ptr [si],'Z'
       ja short lowercase_try_again
       add byte ptr [si],'a'-'A'
       jmp short lowercase_try_again
lowercase_finish_conv:
       ret

; bin2dec (written by Terje Mathisen)
bin2dec:
       mov di,offset buffer              ; convert DX:AX to ASCII string
       push bx                           ;   put ASCII string in buffer
       push cx
       push si

       mov si,dx
       mov bx,10
       mov cx,sp
big:
       cmp dx,bx
       jb short small

       xchg ax,si
       xor dx,dx
       div bx
       xchg ax,si
       div bx
       push dx
       mov dx,si
       jmp short big

small:
       div bx
       push dx
       xor dx,dx
       test ax,ax
       jnz short small

store:
       pop ax
       add al,'0'
       stosb
       cmp cx,sp
       jne short store
       mov byte ptr [di],NULL
       pop si
       pop cx
       pop bx
store_ret:
       ret

; spaces
spaces:
       mov cx,20                         ; align printing output neatly
       mov bl,byte ptr [strlen]
       sub cl,bl
       call space_out                    ; space_out does the dirty work
spaces_ret:
       ret

; nspaces
nspaces:
       test byte ptr [options],OPT_D
       jz short nspaces_no_opt_d
       test byte ptr [dta+FATTR],DIR
       jz short nspaces_ret
nspaces_no_opt_d:
       cmp word ptr [numitems],0
       jz short nspaces_ret
       mov cl,18                         ; align printing properly
       mov al,byte ptr [numlen]
       sub cl,al
       call space_out                    ; space_out to the rescue
nspaces_ret:
       ret

; put_commas
put_commas:
       mov si,offset buffer              ; backup now holds the comma'd number
                                         ;    string for us to print
       mov al,NULL
       mov cx,12
       mov dl,cl
       mov di,offset buffer
       repnz scasb
       mov di,offset backup
       inc cl
       mov al,cl
       sub dl,al
       xchg dl,al
       mov byte ptr [numlen],al
       cmp al,3
       jb short put_commas_small_numbers
       mov cl,3
       div cl
       mov bl,al
       or ah,ah
       jz short put_commas_continue
       mov cl,ah
       rep movsb
       mov bl,al
       mov al,','
       stosb
       inc byte ptr [numlen]
put_commas_continue:
       mov cl,bl
put_commas_loop_here:
       push cx
       mov cx,3
       rep movsb
       mov al,','
       stosb
       inc byte ptr [numlen]
       pop cx
       loop put_commas_loop_here
       dec di
       mov byte ptr [di],NULL
       dec byte ptr [numlen]
       jmp short put_commas_ret
put_commas_small_numbers:
       mov cl,al
       rep movsb
       mov byte ptr [di],NULL
put_commas_ret:
       ret

; scan
scan:
       mov si,offset filespec
       mov di,offset fsbackup
       mov cx,40
       rep movsw                         ; backup our original filespec
       mov al,NULL
       mov di,offset filespec
       cmp byte ptr [di],0
       jz short scan_dec_di
       mov cx,80
       repnz scasb
       sub di,2
       cmp byte ptr [di],'\'
       jz short scan_load_all
       inc di
       mov byte ptr [di],'\'
       jmp short scan_load_all
scan_dec_di:
       dec di
scan_load_all:
       inc di
       mov si,offset all
       mov cx,4
       rep movsb                         ; append '*.*' if last char is '\'
       mov byte ptr [fslen],3            ;    otherwise, append '\*.*'
       mov byte ptr [di],NULL
scan_ret:
       ret

; extra_info
extra_info:
       mov si,offset crlf
       call print
       xor dx,dx
       mov ax,word ptr [numitems]
       call bin2dec
       call put_commas
       mov si,offset backup
       call print                        ; print number of items found
       mov si,offset items_found
       call print
       xor dx,dx
       mov ax,word ptr [numdirs]
       call bin2dec
       call put_commas                   ; print number of dirs found
       mov si,offset backup
       call print
       mov si,offset dirs_found
       call print
       xor dx,dx
       mov ax,word ptr [numfiles]
       call bin2dec
       call put_commas
       mov si,offset backup
       call print                        ; print number of files found
       mov si,offset files_found
       call print
       mov ax,word ptr [total]
       mov dx,word ptr [total+2]
       call bin2dec
       call put_commas
       mov si,offset backup
       call print                        ; print 'totaling --- bytes'
       mov si,offset bytes_msg
       call print
       mov si,offset crlf
       call print
       mov ah,36h
       mov dl,byte ptr [drive]
       int 21h
       mul bx
       mul cx
       call bin2dec
       call put_commas
       mov si,offset backup
       call print                        ; print free space on drive
       mov si,offset space_msg
       call print
       mov si,offset crlf
       call print
       call time_date                    ; print current time and date
       mov si,offset crlf
       call print
extra_info_ret:
       ret

; time_date
time_date:
       mov ah,2Ch                        ; int 21h,2Ch - get current time
       int 21h
       mov si,dx
       xor ah,ah
       mov al,ch                         ; CH = current hour
       or al,al
       jz short time_date_add_twelve
       cmp al,12
       jb short time_date_divide
       mov di,1
       cmp al,12
       jz short time_date_divide
time_date_sub_twelve:                    ; 10:00pm is better than 22:00
       mov bl,12
       sub al,bl
       jmp short time_date_divide
time_date_add_twelve:                    ; 12:00am is better than 00:00
       mov bl,12
       add al,bl
time_date_divide:
       mov bl,10                         ; convert to decimal
       div bl                            ; quotient in AL, remainder in AH
time_date_hours:
       push ax
       mov ah,2
       mov dl,al                         ; int 21h,2 : DL = char to print
       or dl,dl
       jz short time_date_second_digit
       or dl,'0'                         ; convert to ASCII
       int 21h                           ; print first digit
time_date_second_digit:
       pop ax
       mov dl,ah
       or dl,'0'                         ; convert to ASCII
       mov ah,2
       int 21h                           ; print second digit
       mov dl,':'
       int 21h                           ; print colon
time_date_minutes:
       xor ah,ah
       mov al,cl                         ; CL = minutes past current hour
       mov bl,10                         ; convert to base 10 (decimal)
       div bl
       push ax                           ; save results in AX
       mov ah,2
       mov dl,al
       or dl,'0'                         ; ASCIIfy the first digit
       int 21h                           ; print it
       pop ax                            ; restore AX
       mov dl,ah
       mov ah,2
       or dl,'0'                         ; ASCIIfy the second digit
       int 21h                           ; print it
time_date_phase_print:
       cmp di,1
       jnz short time_date_ante_meridian
       mov si,offset pm_msg              ; 'pm'
       call print
       jmp short time_date_continue
time_date_ante_meridian:
       mov si,offset am_msg              ; 'am'
       call print
time_date_continue:
       mov si,offset on_msg              ; 'on'
       call print
time_date_date_time:
       mov ah,2Ah                        ; int 21h,2Ah - get current date
       int 21h
time_date_month:
       push dx                           ; DH = month, DL = day, CX = year
       xor ah,ah
       mov al,dh
       xor dx,dx
       call bin2dec                      ; prints current month
       mov si,offset buffer
       call print
       mov ah,2
       mov dl,'-'
       int 21h
time_date_day:
       pop dx
       xor ah,ah
       mov al,dl
       xor dx,dx
       call bin2dec                      ; prints current day
       mov si,offset buffer
       call print
       mov ah,2
       mov dl,'-'
       int 21h
time_date_year:
       mov ax,cx
       xor dx,dx
       call bin2dec                      ; prints current year
       mov si,offset buffer
       call print
time_date_return:
       ret

; verbose
verbose:                                 ; /V
       mov ax,word ptr [dta+FDATE]
       mov bx,ax
       and ax,0000000111100000b
       mov cl,5
       shr ax,cl
       xor dx,dx
       push ax
       call bin2dec
       pop ax
       call vcheck_ten
       mov si,offset buffer
       call print                        ; print month of filedate
       mov ah,2
       mov dl,'-'
       int 21h
       mov ax,bx
       xor ah,ah
       and al,00011111b
       xor dx,dx
       push ax
       call bin2dec
       pop ax
       call vcheck_ten
       mov si,offset buffer
       call print                        ; print day of filedate
       mov ah,2
       mov dl,'-'
       int 21h
       mov ax,bx
       mov cl,9
       shr ax,cl
       add ax,1980
       xor dx,dx
       call bin2dec
       mov si,offset buffer
       call print                        ; print year of filedate
       xor ch,ch
       mov cl,byte ptr [spacenum]
       call space_out
       mov byte ptr [spacenum],5

       mov di,offset pm_msg
       mov ax,word ptr [dta+FTIME]
       mov bx,ax
       mov al,ah
       xor ah,ah
       mov cl,3
       shr al,cl
       xor dx,dx
       or al,al
       jz short verbose_midnight
       cmp al,12
       jb short verbose_daytime
       mov cl,12
       sub al,cl
       or al,al
       jnz short verbose_print_hour
       add al,12
       jmp short verbose_print_hour
verbose_midnight:
       mov al,12
verbose_daytime:
       mov di,offset am_msg
verbose_print_hour:
       push ax
       push di
       call bin2dec
       pop di
       pop ax
       call vcheck_ten
       mov si,offset buffer
       call print                        ; print hour of filetime
       mov ah,2
       mov dl,':'
       int 21h
verbose_minutes:
       mov ax,bx
       and ah,00000111b
       mov cl,5
       shr ax,cl
       push ax
       push dx
       call check_ten
       pop dx
       pop ax
verbose_print_minutes:
       xor dx,dx
       push di
       call bin2dec
       pop di
       mov si,offset buffer
       call print                        ; print minutes of filetime
       mov ah,2
       mov dl,':'
       int 21h
verbose_seconds:
       mov ax,bx
       xor ah,ah
       and al,00011111b
       add al,al
       push ax
       push dx
       call check_ten
       pop dx
       pop ax
verbose_print_seconds:
       xor dx,dx
       push di
       call bin2dec
       pop di
       mov si,offset buffer
       call print                        ; print seconds of filetime
       mov si,di
       call print
       xor ch,ch
       mov cl,byte ptr [spacenum]
       call space_out
       mov byte ptr [spacenum],5

       xor ah,ah
       mov al,byte ptr [dta+FATTR]
       mov bx,ax
verbose_archive:
       test ax,ARCHIVE
       jz short verbose_not_archive
       mov ah,2
       mov dl,'a'                        ; if file is archive, print 'a'
       int 21h                           ;    else print '.'
       jmp short verbose_system
verbose_not_archive:
       mov ah,2
       mov dl,'.'
       int 21h
verbose_system:
       mov ax,bx
       test ax,SYSTEM
       jz short verbose_not_system
       mov ah,2
       mov dl,'s'                        ; if file is system, print 's'
       int 21h                           ;    else print '.'
       jmp short verbose_hidden
verbose_not_system:
       mov ah,2
       mov dl,'.'
       int 21h
verbose_hidden:
       mov ax,bx
       test ax,HIDDEN
       jz short verbose_not_hidden
       mov ah,2
       mov dl,'h'                        ; if file is hidden, print 'h'
       int 21h                           ;    else print '.'
       jmp short verbose_readonly
verbose_not_hidden:
       mov ah,2
       mov dl,'.'
       int 21h
verbose_readonly:
       mov ax,bx
       test ax,READONLY
       jz short verbose_not_readonly
       mov ah,2
       mov dl,'r'                        ; if file is readonly, print 'r'
       int 21h                           ;    else print '.'
       jmp short verbose_ret
verbose_not_readonly:
       mov ah,2
       mov dl,'.'
       int 21h
verbose_ret:
       ret

; vcheck_ten
vcheck_ten:                              ; if /Z, ignore leading zeros in /V
       test byte ptr [options],OPT_Z
       jnz short vcheck_ten_space_adjust
       cmp ax,10
       jae short vcheck_ten_ret
       mov ah,2
       mov dl,'0'
       int 21h
       jmp short vcheck_ten_ret
vcheck_ten_space_adjust:
       cmp ax,10
       jae short vcheck_ten_ret
       inc byte ptr [spacenum]
vcheck_ten_ret:
       ret

; check_ten
check_ten:                               ; same as vcheck_ten, but always
       cmp ax,10                         ;    prints leading zeros
       jae short check_ten_ret
       mov ah,2
       mov dl,'0'
       int 21h
check_ten_ret:
       ret

; space_out
space_out:                               ; CX = number of spaces to print
       mov ah,2
       mov dl,' '
space_out_print_it:
       int 21h
       loop space_out_print_it
space_out_ret:
       ret

; check_filespec
check_filespec:                          ; finds drive that is being searched
      mov al,':'
      xor ch,ch
      mov cl,byte ptr [fslen]
      mov di,offset filespec
      repnz scasb
      jcxz check_filespec_get_cwd
      mov cx,2
      sub di,cx
      xchg si,di
      lodsb
      cmp al,'A'
      jb short check_filespec_fix
      cmp al,'Z'
      ja short check_filespec_fix
check_filespec_fix:
      cmp al,'a'
      jb short check_filespec_convert
      cmp al,'z'
      ja short check_filespec_convert
check_filespec_okay:
      mov cl,'a'
      sub al,cl
      inc al
      mov byte ptr [drive],al            ; remember current drive, if user
      jmp check_filespec_ret             ;    mentioned one in filespec
check_filespec_convert:
      add al,'a'-'A'
      jmp short check_filespec_okay
check_filespec_get_cwd:
      mov ah,19h
      int 21h                            ; get current drive
      mov dl,al
      cmp word ptr [filespec],'..'
      jnz short check_filespec_no_amend  ; all the following is to
      inc dl                             ;   make sure the correct
      add al,'A'                         ;   filespec is printed
      mov di,offset filespec             ;   if we type 'dw ..' or
      stosb                              ;   something similar to
      mov al,':'                         ;   'dw ..\*.exe'
      stosb
      mov bx,di                          ; we typed '..' and called scan,
      mov al,NULL                        ;   making it '..\*.*', then
      mov cx,15                          ;   replaced '..' with current
      repnz scasb                        ;   drive and following colon
      xchg bx,di                         ;   (e.g. 'C:\*.*')
      sub bx,di                          ; DI now points to the '\' in 'C:\*.*'
      mov ah,47h                         ; BX = length of '\*.*'
      mov si,offset curdir
      int 21h                            ; get the current directory
      xchg si,di
      mov al,NULL
      mov cx,64
      repnz scasb
      dec di
      dec di
      mov al,'\'
      mov cx,15
      std
      repnz scasb                        ; find the last '\' in current dir
      cld
      jcxz check_filespec_ret
      inc di
      mov cx,bx                          ; if we're in 'C:\TEMP\ASM',
      rep movsb                          ;   we remove the '\ASM' and
      mov cx,offset curdir               ;   then append the '\*.*'
      xchg cx,di                         ;   from above to it
      sub cx,di                          ; our new filespec is the following:
      mov si,offset curdir               ;   'C:\TEMP\*.*' (or similar)
      mov di,offset filespec             ; sorry for the confusion  ;-)
      add di,3
      rep movsb
      jmp short check_filespec_ret
check_filespec_no_amend:
      add dl,'A'
      mov ah,2
      int 21h
      mov dl,':'
      int 21h
      cmp byte ptr [filespec],'\'
      jz short check_filespec_ret
      mov dl,'\'
      int 21h
      mov ah,47h                         ; get current directory
      mov dl,byte ptr [drive]
      mov si,offset curdir
      int 21h
      call print                         ; print current dir before filespec,
      mov di,offset curdir               ;    if needed
      cmp byte ptr [di],0
      jz short check_filespec_ret
      mov ah,2
      mov dl,'\'
      int 21h
check_filespec_ret:
      ret

; add_total
add_total:                               ; used with /I only
      test byte ptr [options],OPT_I
      jnz short add_total_ret
      mov ax,word ptr [total]
      mov bx,word ptr [dta+FSIZEL]
      add ax,bx
      mov word ptr [total],ax
      mov ax,word ptr [total+2]
      mov bx,word ptr [dta+FSIZEH]
      adc ax,bx
      mov word ptr [total+2],ax          ; this keeps a total size (in bytes)
add_total_ret:                           ;    of all files found
      ret

; default
default:
      xor byte ptr [options],OPT_I       ; set default options
default_ret:
      ret

; print_filespec
print_filespec:
      test byte ptr [options],OPT_L
      jnz short print_filespec_ret
      mov ah,2
      mov dl,'('
      int 21h
      call check_filespec
      mov si,offset filespec
      call print                         ; print filespec
      mov ah,2
      mov dl,')'
      int 21h
      mov si,offset crlf
      call print
print_filespec_ret:
      ret

[section .data]

  all db '*.*',0                         ; default filespec
  crlf db CR,LF,NULL                     ; newline
  dir_msg db '<dir>',NULL
  no_files_found db CR,LF,'No files found.',CR,LF,NULL
  invalid_path db CR,LF,'Invalid path.',CR,LF,NULL
  items_found db ' items: ',NULL
  files_found db ' files totaling ',NULL
  dirs_found db ' dirs, ',NULL
  bytes_msg db ' bytes',NULL
  space_msg db ' bytes free',NULL
  on_msg db ' on ',NULL
  am_msg db 'am',NULL
  pm_msg db 'pm',NULL

  help_msg db CR,LF,'DirWide 1.8 - anthonyw@dibbs.net',CR,LF
           db CR,LF,'Usage: dw [filespec] [options]',CR,LF
           db '/c no commas',CR,LF
           db '/d dirs',CR,LF
           db '/f files',CR,LF
           db '/h this screen',CR,LF
           db '/i misc info',CR,LF
           db '/l no filespec printing',CR,LF
           db '/p no pause',CR,LF
           db '/v verbose item info',CR,LF
           db '/z no lead zeros with /v',CR,LF,NULL

  pausenum db 30                         ; pause after this # of files if found
  spacenum db 5                          ; number of spaces for /V
  total dd 0                             ; total of filesizes found
  numitems dw 0                          ; number of items found
  numfiles dw 0                          ; number of files found
  numdirs dw 0                           ; number of dirs found
  fslen db 0                             ; filespec length
  drive db 0                             ; drive being searched
  attribute db 0                         ; (with /F or /D only)
  options db 0                           ; our options

[section .bss]

  curdir resb 64
  dta resb 43
  buffer resb 12
  backup resb 16
  filespec resb 80
  fsbackup resb 80
  strlen resb 1
  numlen resb 1

  ; curdir db 64 dup(?)
  ; dta db 43 dup(?)
  ; buffer db 12 dup(?)
  ; backup db 16 dup(?)
  ; filespec db 80 dup(?)
  ; fsbackup db 80 dup(?)
  ; strlen db ?
  ; numlen db ?

; CODE ends
; end Start
